@@ -32,7 +32,7 @@
 (*                                                                         *)
 (***************************************************************************)
 
-(* $Id: content2pres.ml 7922 2007-11-25 13:12:25Z tassi $ *)
+(* $Id: content2pres.ml 8611 2008-05-21 12:23:53Z tassi $ *)
 
 module P = Mpresentation
 module B = Box
@@ -121,7 +121,7 @@ let add_xref id = function
   | B.Text (attrs, t) -> B.Text (((Some "helm", "xref", id) :: attrs), t)
   | _ -> assert false (* TODO, add_xref is meaningful for all boxes *)
 
-let rec justification ~ignore_atoms term2pres p = 
+let rec justification ~for_rewriting_step ~ignore_atoms term2pres p = 
   if p.Con.proof_conclude.Con.conclude_method = "Exact" &&
      ignore_atoms
   then
@@ -136,10 +136,16 @@ let rec justification ~ignore_atoms term2pres p =
       make_args_for_apply term2pres p.Con.proof_conclude.Con.conclude_args
     in
      [B.H([],
-       (B.b_kw "by")::B.b_space::
+       (if for_rewriting_step then (B.b_kw "exact") else (B.b_kw "by"))::
+       B.b_space::
        B.Text([],"(")::pres_args@[B.Text([],")")])], None 
   else
-   [B.H([],[B.b_kw "by"; B.b_space; B.b_kw "proof"])],
+   [B.H([],
+    if for_rewriting_step then
+     [B.b_kw "proof"]
+    else
+     [B.b_kw "by"; B.b_space; B.b_kw "proof"]
+    )],
     Some (B.b_toggle [B.b_kw "proof";B.indent (proof2pres true term2pres p)])
      
 and proof2pres ?skip_initial_lambdas is_top_down term2pres p =
@@ -441,7 +447,9 @@ let body = B.V([],[B.b_kw ("(*<<" ^ p.Con.proof_conclude.Con.conclude_method ^ (
          || conclude.Con.conclude_method = "RewriteRL" then
       let justif1,justif2 = 
         (match (List.nth conclude.Con.conclude_args 6) with
-           Con.ArgProof p -> justification ~ignore_atoms:true term2pres p
+           Con.ArgProof p ->
+            justification ~for_rewriting_step:true ~ignore_atoms:true
+             term2pres p
          | _ -> assert false) in
       let justif =
        match justif2 with
@@ -483,7 +491,9 @@ let body = B.V([],[B.b_kw ("(*<<" ^ p.Con.proof_conclude.Con.conclude_method ^ (
       B.V([], justif @ [B.b_kw "by _"])
     else if conclude.Con.conclude_method = "Eq_chain" then
       let justification p =
-       let j1,j2 = justification ~ignore_atoms:false term2pres p in
+       let j1,j2 =
+        justification ~for_rewriting_step:true ~ignore_atoms:false term2pres p
+       in
         j1, match j2 with Some j -> [j] | None -> []
       in
       let rec aux args =
@@ -885,8 +895,10 @@ let recursion_kind2pres params kind =
     match kind with
     | `Recursive _ -> "Recursive definition"
     | `CoRecursive -> "CoRecursive definition"
-    | `Inductive _ -> "Inductive definition"
-    | `CoInductive _ -> "CoInductive definition"
+    | `Inductive i -> 
+        "Inductive definition with "^string_of_int i^" fixed parameter(s)"
+    | `CoInductive i -> 
+        "Co-Inductive definition with "^string_of_int i^" fixed parameter(s)"
   in
   B.b_h [] (B.b_kw kind :: params2pres params)
 
