@@ -28,13 +28,8 @@ exception UnableToInclude of string
   (* statements meaningful for matitadep *)
 type dependency =
   | IncludeDep of string
-  | BaseuriDep of string
   | UriDep of UriManager.uri
 
 val pp_dependency: dependency -> string
 
-  (** @raise End_of_file *)
-val parse_dependencies: Ulexing.lexbuf -> dependency list
-
-(* returns baseuri and the full path of the script *)
-val baseuri_of_script : include_paths:string list -> string -> string * string
+val deps_of_file: string -> dependency list
