@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: grafiteDisambiguate.ml 7922 2007-11-25 13:12:25Z tassi $ *)
+(* $Id: grafiteDisambiguate.ml 8754 2008-06-24 14:47:16Z tassi $ *)
 
 exception BaseUriNotSetYet
 
@@ -36,16 +36,20 @@ type lazy_tactic =
   (Cic.term, Cic.lazy_term, Cic.lazy_term GrafiteAst.reduction, string) 
     GrafiteAst.tactic
 
-let singleton = function
+let singleton msg = function
   | [x], _ -> x
-  | _ -> assert false
-;;
+  | l, _   ->
+      let debug = 
+         Printf.sprintf "GrafiteDisambiguate.singleton (%s): %u interpretations"
+	 msg (List.length l)
+      in
+      HLog.debug debug; assert false
 
   (** @param term not meaningful when context is given *)
 let disambiguate_term text prefix_len lexicon_status_ref context metasenv term =
   let lexicon_status = !lexicon_status_ref in
   let (diff, metasenv, cic, _) =
-    singleton
+    singleton "first"
       (GrafiteDisambiguator.disambiguate_term ~dbd:(LibraryDb.instance ())
         ~aliases:lexicon_status.LexiconEngine.aliases
         ~universe:(Some lexicon_status.LexiconEngine.multi_aliases)
@@ -65,7 +69,7 @@ let disambiguate_lazy_term text prefix_len lexicon_status_ref term =
   (fun context metasenv ugraph ->
     let lexicon_status = !lexicon_status_ref in
     let (diff, metasenv, cic, ugraph) =
-      singleton
+      singleton "second"
         (GrafiteDisambiguator.disambiguate_term ~dbd:(LibraryDb.instance ())
           ~initial_ugraph:ugraph ~aliases:lexicon_status.LexiconEngine.aliases
           ~universe:(Some lexicon_status.LexiconEngine.multi_aliases)
@@ -99,12 +103,35 @@ let disambiguate_reduction_kind text prefix_len lexicon_status_ref = function
       let t = disambiguate_lazy_term text prefix_len lexicon_status_ref t in
       `Unfold (Some t)
   | `Normalize
-  | `Reduce
   | `Simpl
   | `Unfold None
   | `Whd as kind -> kind
 ;;
 
+let disambiguate_auto_params 
+  disambiguate_term metasenv context (terms, params) 
+=
+    let metasenv, terms = 
+      List.fold_right 
+       (fun t (metasenv, terms) ->
+         let metasenv,t = disambiguate_term context metasenv t in
+         metasenv,t::terms) terms (metasenv, [])
+    in
+    metasenv, (terms, params)
+;;
+
+let disambiguate_just disambiguate_term context metasenv =
+ function
+    `Term t ->
+      let metasenv,t = disambiguate_term context metasenv t in
+       metasenv, `Term t
+  | `Auto params ->
+      let metasenv,params = disambiguate_auto_params disambiguate_term metasenv
+       context params
+      in
+       metasenv, `Auto params
+;;
+      
 let rec disambiguate_tactic 
   lexicon_status_ref context metasenv (text,prefix_len,tactic) 
 =
@@ -118,6 +145,9 @@ let rec disambiguate_tactic
     disambiguate_lazy_term text prefix_len lexicon_status_ref in
   let disambiguate_tactic metasenv tac =
    disambiguate_tactic lexicon_status_ref context metasenv (text,prefix_len,tac)
+  in
+  let disambiguate_auto_params m p = 
+    disambiguate_auto_params disambiguate_term m context p
   in
    match tactic with
     (* Higher  order tactics *)
@@ -178,15 +208,18 @@ let rec disambiguate_tactic
         let metasenv,cic = disambiguate_term context metasenv term in
         metasenv,GrafiteAst.Apply (loc, cic)
     | GrafiteAst.ApplyS (loc, term, params) ->
+        let metasenv, params = disambiguate_auto_params metasenv params in
         let metasenv,cic = disambiguate_term context metasenv term in
         metasenv,GrafiteAst.ApplyS (loc, cic, params)
     | GrafiteAst.Assumption loc ->
         metasenv,GrafiteAst.Assumption loc
     | GrafiteAst.AutoBatch (loc,params) ->
+        let metasenv, params = disambiguate_auto_params metasenv params in
         metasenv,GrafiteAst.AutoBatch (loc,params)
-    | GrafiteAst.Cases (loc, what, idents) ->
+    | GrafiteAst.Cases (loc, what, pattern, idents) ->
         let metasenv,what = disambiguate_term context metasenv what in
-        metasenv,GrafiteAst.Cases (loc, what, idents)
+	let pattern = disambiguate_pattern pattern in
+        metasenv,GrafiteAst.Cases (loc, what, pattern, idents)
     | GrafiteAst.Change (loc, pattern, with_what) -> 
         let with_what = disambiguate_lazy_term with_what in
         let pattern = disambiguate_pattern pattern in
@@ -214,8 +247,9 @@ let rec disambiguate_tactic
         metasenv,GrafiteAst.Cut (loc, ident, cic)
     | GrafiteAst.Decompose (loc, names) ->
          metasenv,GrafiteAst.Decompose (loc, names)
-    | GrafiteAst.Demodulate loc ->
-        metasenv,GrafiteAst.Demodulate loc
+    | GrafiteAst.Demodulate (loc, params) ->
+        let metasenv, params = disambiguate_auto_params metasenv params in
+        metasenv,GrafiteAst.Demodulate (loc, params)
     | GrafiteAst.Destruct (loc, Some terms) ->
         let map term (metasenv, terms) =
            let metasenv, term = disambiguate_term context metasenv term in
@@ -318,14 +352,11 @@ let rec disambiguate_tactic
 	          let metasenv,t = disambiguate_term context metasenv t in
 		  metasenv,Some t in
 	metasenv,GrafiteAst.Suppose (loc, cic, id, cic')
-    | GrafiteAst.Bydone (loc,term) ->
-        let metasenv,cic = 
-	   match term with
-	      None -> metasenv,None
-	     |Some t ->
-                 let metasenv,t = disambiguate_term context metasenv t in
-		 metasenv,Some t in
-	         metasenv,GrafiteAst.Bydone (loc, cic)
+    | GrafiteAst.Bydone (loc,just) ->
+        let metasenv,just =
+         disambiguate_just disambiguate_term context metasenv just
+        in
+	 metasenv,GrafiteAst.Bydone (loc, just)
     | GrafiteAst.We_need_to_prove (loc,term,id,term') ->
         let metasenv,cic = disambiguate_term context metasenv term in
 	let metasenv,cic' = 
@@ -335,13 +366,9 @@ let rec disambiguate_tactic
 	          let metasenv,t = disambiguate_term context metasenv t in
 		  metasenv,Some t in
 	metasenv,GrafiteAst.We_need_to_prove (loc,cic,id,cic')
-    | GrafiteAst.By_term_we_proved (loc,term,term',id,term'') ->
-        let metasenv,cic =
-	   match term with 
-	     None -> metasenv,None
-	   | Some t ->
-	         let metasenv,t = disambiguate_term context metasenv t in
-		 metasenv,Some t in
+    | GrafiteAst.By_just_we_proved (loc,just,term',id,term'') ->
+        let metasenv,just =
+         disambiguate_just disambiguate_term context metasenv just in
         let metasenv,cic' = disambiguate_term context metasenv term' in
 	let metasenv,cic'' = 
 	    match term'' with
@@ -349,7 +376,7 @@ let rec disambiguate_tactic
 	   |  Some t ->  
 	            let metasenv,t = disambiguate_term context metasenv t in
 	             metasenv,Some t in
-	metasenv,GrafiteAst.By_term_we_proved (loc,cic,cic',id,cic'')
+	metasenv,GrafiteAst.By_just_we_proved (loc,just,cic',id,cic'')
     | GrafiteAst.We_proceed_by_cases_on (loc, term, term') ->
         let metasenv,cic = disambiguate_term context metasenv term in
 	let metasenv,cic' = disambiguate_term context metasenv term' in
@@ -364,21 +391,18 @@ let rec disambiguate_tactic
    | GrafiteAst.Thesisbecomes (loc, term) ->
         let metasenv,cic = disambiguate_term context metasenv term in
 	metasenv,GrafiteAst.Thesisbecomes (loc, cic)
-   | GrafiteAst.ExistsElim (loc, term, id1, term1, id2, term2) ->
-	let metasenv,cic =
-           match term with
-             None -> metasenv,None
-           | Some t ->
-                 let metasenv,t = disambiguate_term context metasenv t in
-                 metasenv,Some t in
+   | GrafiteAst.ExistsElim (loc, just, id1, term1, id2, term2) ->
+	let metasenv,just =
+         disambiguate_just disambiguate_term context metasenv just in
         let metasenv,cic' = disambiguate_term context metasenv term1 in
 	let cic''= disambiguate_lazy_term term2 in
-	metasenv,GrafiteAst.ExistsElim(loc, cic, id1, cic', id2, cic'')
-   | GrafiteAst.AndElim (loc, term, id, term1, id1, term2) ->
-	let metasenv,cic = disambiguate_term context metasenv term in
+	metasenv,GrafiteAst.ExistsElim(loc, just, id1, cic', id2, cic'')
+   | GrafiteAst.AndElim (loc, just, id, term1, id1, term2) ->
+	let metasenv,just =
+         disambiguate_just disambiguate_term context metasenv just in
 	let metasenv,cic'= disambiguate_term context metasenv term1 in
 	let metasenv,cic''= disambiguate_term context metasenv term2 in
-	metasenv,GrafiteAst.AndElim(loc, cic, id, cic', id1, cic'')   
+	metasenv,GrafiteAst.AndElim(loc, just, id, cic', id1, cic'')   
    | GrafiteAst.Case (loc, id, params) ->
         let metasenv,params' =
 	 List.fold_right
@@ -398,7 +422,12 @@ let rec disambiguate_tactic
 	let metasenv,cic'= disambiguate_term context metasenv term2 in
         let metasenv,cic'' =
 	 match term3 with
-	    `Auto _ as t -> metasenv,t
+          | `SolveWith term ->
+	     let metasenv,term = disambiguate_term context metasenv term in
+             metasenv, `SolveWith term
+	  | `Auto params -> 
+              let metasenv, params = disambiguate_auto_params metasenv params in
+              metasenv,`Auto params
 	  | `Term t -> 
 	     let metasenv,t = disambiguate_term context metasenv t in
 	      metasenv,`Term t
@@ -406,7 +435,7 @@ let rec disambiguate_tactic
 	metasenv,GrafiteAst.RewritingStep (loc, cic, cic', cic'', cont)   
 
 
-let disambiguate_obj lexicon_status ~baseuri metasenv (text,prefix_len,obj) =
+let disambiguate_obj lexicon_status ?baseuri metasenv (text,prefix_len,obj) =
   let uri =
    match obj with
     | CicNotationPt.Inductive (_,(name,_,_,_)::_)
@@ -418,7 +447,7 @@ let disambiguate_obj lexicon_status ~baseuri metasenv (text,prefix_len,obj) =
     | CicNotationPt.Inductive _ -> assert false
     | CicNotationPt.Theorem _ -> None in
   let (diff, metasenv, cic, _) =
-    singleton
+    singleton "third"
       (GrafiteDisambiguator.disambiguate_obj ~dbd:(LibraryDb.instance ())
         ~aliases:lexicon_status.LexiconEngine.aliases
         ~universe:(Some lexicon_status.LexiconEngine.multi_aliases) ~uri 
@@ -426,7 +455,7 @@ let disambiguate_obj lexicon_status ~baseuri metasenv (text,prefix_len,obj) =
   let lexicon_status = LexiconEngine.set_proof_aliases lexicon_status diff in
   lexicon_status, metasenv, cic
   
-let disambiguate_command lexicon_status ~baseuri metasenv (text,prefix_len,cmd)=
+let disambiguate_command lexicon_status ?baseuri metasenv (text,prefix_len,cmd)=
   match cmd with
    | GrafiteAst.Index(loc,key,uri) ->
        let lexicon_status_ref = ref lexicon_status in 
@@ -441,7 +470,12 @@ let disambiguate_command lexicon_status ~baseuri metasenv (text,prefix_len,cmd)=
        in
        let metasenv,key = disambiguate_term_option metasenv key in
        !lexicon_status_ref, metasenv,GrafiteAst.Index(loc,key,uri)
-   | GrafiteAst.Coercion _
+   | GrafiteAst.Coercion (loc,t,b,a,s) -> 
+       let lexicon_status_ref = ref lexicon_status in 
+       let disambiguate_term =
+	 disambiguate_term text prefix_len lexicon_status_ref [] in
+      let metasenv,t = disambiguate_term metasenv t in
+      !lexicon_status_ref, metasenv, GrafiteAst.Coercion (loc,t,b,a,s)
    | GrafiteAst.Default _
    | GrafiteAst.Drop _
    | GrafiteAst.Include _
@@ -451,7 +485,7 @@ let disambiguate_command lexicon_status ~baseuri metasenv (text,prefix_len,cmd)=
        lexicon_status,metasenv,cmd
    | GrafiteAst.Obj (loc,obj) ->
        let lexicon_status,metasenv,obj =
-        disambiguate_obj lexicon_status ~baseuri metasenv (text,prefix_len,obj)in
+        disambiguate_obj lexicon_status ?baseuri metasenv (text,prefix_len,obj)in
        lexicon_status, metasenv, GrafiteAst.Obj (loc,obj)
    | GrafiteAst.Relation (loc,id,a,aeq,refl,sym,trans) ->
       let lexicon_status_ref = ref lexicon_status in 
@@ -492,7 +526,10 @@ let disambiguate_macro
    | GrafiteAst.Check (loc,term) ->
       let metasenv,term = disambiguate_term context metasenv term in
        metasenv,GrafiteAst.Check (loc,term)
-   | GrafiteAst.AutoInteractive _
+   | GrafiteAst.AutoInteractive (loc, params) -> 
+      let metasenv, params = 
+        disambiguate_auto_params disambiguate_term metasenv context params in
+      metasenv, GrafiteAst.AutoInteractive (loc, params)
    | GrafiteAst.Hint _
    | GrafiteAst.WLocate _
    | GrafiteAst.Inline _ as macro ->
