@@ -461,7 +461,7 @@ let close_coercion_graph src tgt uri saturations baseuri =
                         let o,univ = build_obj t univ arityres in
                          (o,saturationsres,arityres),univ
                     | _ -> assert false 
-                  ) (first_step, CicUniv.empty_ugraph) tl
+                  ) (first_step, CicUniv.oblivion_ugraph) tl
                 in
                 let name_src = CoercDb.name_of_carr src in
                 let name_tgt = CoercDb.name_of_carr tgt in
