@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: primitiveTactics.ml 7922 2007-11-25 13:12:25Z tassi $ *)
+(* $Id: primitiveTactics.ml 8737 2008-06-19 16:34:41Z sacerdot $ *)
 
 exception TheTypeOfTheCurrentGoalIsAMetaICannotChooseTheRightElimiantionPrinciple
 exception NotAnInductiveTypeToEliminate
@@ -62,11 +62,11 @@ let lambda_abstract ?(howmany=(-1)) metasenv context newmeta ty mk_fresh_name =
            collect_context ctx (howmany - 1) do_whd t 
           in
            (context',ty,C.Lambda(n',s,bo))
-      | C.LetIn (n,s,t) ->
+      | C.LetIn (n,s,sty,t) ->
          let (context',ty,bo) =
-          collect_context ((Some (n,(C.Def (s,None))))::context) (howmany - 1) do_whd t
+          collect_context ((Some (n,(C.Def (s,sty))))::context) (howmany - 1) do_whd t
          in
-          (context',ty,C.LetIn(n,s,bo))
+          (context',ty,C.LetIn(n,s,sty,bo))
       | _ as t ->
         if howmany <= 0 then
          let irl =
@@ -102,7 +102,7 @@ let eta_expand metasenv context t arg =
     | C.Cast (te,ty) -> C.Cast (aux n te, aux n ty)
     | C.Prod (nn,s,t) -> C.Prod (nn, aux n s, aux (n+1) t)
     | C.Lambda (nn,s,t) -> C.Lambda (nn, aux n s, aux (n+1) t)
-    | C.LetIn (nn,s,t) -> C.LetIn (nn, aux n s, aux (n+1) t)
+    | C.LetIn (nn,s,ty,t) -> C.LetIn (nn, aux n s, aux n ty, aux (n+1) t)
     | C.Appl l -> C.Appl (List.map (aux n) l)
     | C.Const (uri,exp_named_subst) ->
        let exp_named_subst' = aux_exp_named_subst n exp_named_subst in
@@ -136,7 +136,7 @@ let eta_expand metasenv context t arg =
    List.map (function uri,t -> uri,aux n t)
   in
    let argty,_ = 
-    T.type_of_aux' metasenv context arg CicUniv.empty_ugraph (* TASSI: FIXME *)
+    T.type_of_aux' metasenv context arg CicUniv.oblivion_ugraph (* TASSI: FIXME *)
    in
     let fresh_name =
      FreshNamesGenerator.mk_fresh_name ~subst:[]
@@ -159,15 +159,13 @@ let classify_metas newmeta in_subst_domain subst_in metasenv =
            match entry with
               Some (n,Cic.Decl s) ->
                Some (n,Cic.Decl (subst_in canonical_context' s))
-            | Some (n,Cic.Def (s,None)) ->
-               Some (n,Cic.Def ((subst_in canonical_context' s),None))
             | None -> None
-            | Some (n,Cic.Def (bo,Some ty)) ->
+            | Some (n,Cic.Def (bo,ty)) ->
                Some
                 (n,
                   Cic.Def
                    (subst_in canonical_context' bo,
-                    Some (subst_in canonical_context' ty)))
+                    subst_in canonical_context' ty))
           in
            entry'::canonical_context'
         ) canonical_context []
@@ -184,7 +182,7 @@ let
 =
  let module C = Cic in
   let params =
-    let o,_ = CicEnvironment.get_obj CicUniv.empty_ugraph uri in
+    let o,_ = CicEnvironment.get_obj CicUniv.oblivion_ugraph uri in
     CicUtil.params_of_obj o
   in
    let exp_named_subst_diff,new_fresh_meta,newmetasenvfragment,exp_named_subst'=
@@ -196,7 +194,7 @@ let
          [],[] -> []
        | uri::tl,[] ->
           let ty =
-            let o,_ = CicEnvironment.get_obj CicUniv.empty_ugraph uri in
+            let o,_ = CicEnvironment.get_obj CicUniv.oblivion_ugraph uri in
               match o with
                   C.Variable (_,_,ty,_,_) ->
                     CicSubstitution.subst_vars !exp_named_subst_diff ty
@@ -244,7 +242,7 @@ let new_metasenv_and_unify_and_t newmeta' metasenv' subst context term' ty termt
     goal_arity in
   let subst,newmetasenv',_ = 
    CicUnification.fo_unif_subst 
-     subst context newmetasenv consthead ty CicUniv.empty_ugraph
+     subst context newmetasenv consthead ty CicUniv.oblivion_ugraph
   in
   let t = 
     if List.length arguments = 0 then term' else Cic.Appl (term'::arguments)
@@ -298,7 +296,7 @@ let apply_with_subst ~term ~subst ~maxmeta (proof, goal) =
    in
    let metasenv' = metasenv@newmetasenvfragment in
    let termty,_ = 
-     CicTypeChecker.type_of_aux' metasenv' context term' CicUniv.empty_ugraph
+     CicTypeChecker.type_of_aux' metasenv' context term' CicUniv.oblivion_ugraph
    in
    let termty =
      CicSubstitution.subst_vars exp_named_subst_diff termty in
@@ -409,15 +407,8 @@ let cut_tac ?(mk_fresh_name_callback = FreshNamesGenerator.mk_fresh_name ~subst:
       CicMkImplicit.identity_relocation_list_for_metavariable context
     in
      let newmeta1ty = CicSubstitution.lift 1 ty in
-(* This is the pre-letin implementation
-     let bo' =
-      C.Appl
-       [C.Lambda (fresh_name,term,C.Meta (newmeta1,irl1)) ;
-        C.Meta (newmeta2,irl2)]
-     in
-*)
       let bo' = 
-        Cic.LetIn (fresh_name, C.Meta (newmeta2,irl2), C.Meta (newmeta1,irl1))
+        Cic.LetIn (fresh_name, C.Meta (newmeta2,irl2), term, C.Meta (newmeta1,irl1))
       in
       let (newproof, _) =
        ProofEngineHelpers.subst_meta_in_proof proof metano bo'
@@ -444,19 +435,19 @@ let letin_tac ?(mk_fresh_name_callback=FreshNamesGenerator.mk_fresh_name ~subst:
      raise 
        (ProofEngineTypes.Fail (lazy
          "You can't letin a term containing the current goal"));
-    let _,_ =
-      CicTypeChecker.type_of_aux' metasenv context term CicUniv.empty_ugraph in
+    let tty,_ =
+      CicTypeChecker.type_of_aux' metasenv context term CicUniv.oblivion_ugraph in
      let newmeta = ProofEngineHelpers.new_meta_of_proof ~proof in
      let fresh_name =
       mk_fresh_name_callback metasenv context (Cic.Name "Hletin") ~typ:term in
      let context_for_newmeta =
-      (Some (fresh_name,C.Def (term,None)))::context in
+      (Some (fresh_name,C.Def (term,tty)))::context in
      let irl =
       CicMkImplicit.identity_relocation_list_for_metavariable
        context_for_newmeta
      in
       let newmetaty = CicSubstitution.lift 1 ty in
-      let bo' = C.LetIn (fresh_name,term,C.Meta (newmeta,irl)) in
+      let bo' = C.LetIn (fresh_name,term,tty,C.Meta (newmeta,irl)) in
        let (newproof, _) =
          ProofEngineHelpers.subst_meta_in_proof
            proof metano bo'[newmeta,context_for_newmeta,newmetaty]
@@ -473,7 +464,7 @@ let exact_tac ~term =
   let metano,context,ty = CicUtil.lookup_meta goal metasenv in
   let module T = CicTypeChecker in
   let module R = CicReduction in
-  let ty_term,u = T.type_of_aux' metasenv context term CicUniv.empty_ugraph in
+  let ty_term,u = T.type_of_aux' metasenv context term CicUniv.oblivion_ugraph in
   let b,_ = R.are_convertible context ty_term ty u in (* TASSI: FIXME *)
   if b then
    begin
@@ -523,20 +514,17 @@ let mk_predicate_for_elim
 (* FG: we find the predicate for the eliminator as in the rewrite tactic ****)
 	 let argty, _ugraph = TC.type_of_aux' metasenv context arg ugraph in
          let argty = CicReduction.whd context argty in         
-	       let fresh_name = 
+         let fresh_name = 
             FreshNamesGenerator.mk_fresh_name 
-            ~subst:[] metasenv context' C.Anonymous ~typ:argty
-         in
+            ~subst:[] metasenv context' C.Anonymous ~typ:argty in
 	 let hyp = Some (fresh_name, C.Decl argty) in
          let lazy_term c m u =  
-            let distance  = List.length c - List.length context in
-            S.lift distance arg, m, u
-         in
+          let distance  = List.length c - List.length context in
+           S.lift distance arg, m, u in
          let pattern = Some lazy_term, [], Some cpattern' in
          let subst, metasenv, _ugraph, _conjecture, selected_terms =
-            ProofEngineHelpers.select
-            ~metasenv ~ugraph ~conjecture:(0, context, pred) ~pattern
-         in
+          ProofEngineHelpers.select ~metasenv ~ugraph
+           ~conjecture:(0, context, pred) ~pattern in
          let metasenv = MS.apply_subst_metasenv subst metasenv in  
          let map (_context_of_t, t) l = t :: l in
          let what = List.fold_right map selected_terms [] in
@@ -602,9 +590,9 @@ let beta_after_elim_tac upto predicate =
             | C.Lambda (_, s, t) ->
                let s, t = gen_term k s, gen_term (succ k) t in
                if is_meta [s; t] then meta else C.Lambda (anon, s, t)
-            | C.LetIn (_, s, t) -> 
-               let s, t = gen_term k s, gen_term (succ k) t in
-               if is_meta [s; t] then meta else C.LetIn (anon, s, t)
+            | C.LetIn (_, s, ty, t) -> 
+               let s,ty,t = gen_term k s, gen_term k ty, gen_term (succ k) t in
+               if is_meta [s; t] then meta else C.LetIn (anon, s, ty, t)
             | C.Fix (i, fl) -> C.Fix (i, List.map (gen_fix (List.length fl) k) fl)
             | C.CoFix (i, fl) -> C.CoFix (i, List.map (gen_cofix (List.length fl) k) fl)
          in
@@ -616,17 +604,184 @@ let beta_after_elim_tac upto predicate =
       PET.apply_tactic tactic status
    in
    PET.mk_tactic beta_after_elim_tac
-   
+
+(* ANCORA DA DEBUGGARE *)
+
+exception UnableToDetectTheTermThatMustBeGeneralizedYouMustGiveItExplicitly;;
+exception TheSelectedTermsMustLiveInTheGoalContext
+exception AllSelectedTermsMustBeConvertible;;
+exception GeneralizationInHypothesesNotImplementedYet;;
+
+let generalize_tac 
+ ?(mk_fresh_name_callback = FreshNamesGenerator.mk_fresh_name ~subst:[])
+ pattern
+ =
+  let module PET = ProofEngineTypes in
+  let generalize_tac mk_fresh_name_callback
+       ~pattern:(term,hyps_pat,_) status
+  =
+   if hyps_pat <> [] then raise GeneralizationInHypothesesNotImplementedYet;
+   let (proof, goal) = status in
+   let module C = Cic in
+   let module T = Tacticals in
+    let uri,metasenv,_subst,pbo,pty, attrs = proof in
+    let (_,context,ty) as conjecture = CicUtil.lookup_meta goal metasenv in
+    let subst,metasenv,u,selected_hyps,terms_with_context =
+     ProofEngineHelpers.select ~metasenv ~ugraph:CicUniv.oblivion_ugraph
+      ~conjecture ~pattern in
+    let context = CicMetaSubst.apply_subst_context subst context in
+    let metasenv = CicMetaSubst.apply_subst_metasenv subst metasenv in
+    let pbo = CicMetaSubst.apply_subst subst pbo in
+    let pty = CicMetaSubst.apply_subst subst pty in
+    let term =
+     match term with
+        None -> None
+      | Some term ->
+          Some (fun context metasenv ugraph -> 
+                  let term, metasenv, ugraph = term context metasenv ugraph in
+                   CicMetaSubst.apply_subst subst term,
+                    CicMetaSubst.apply_subst_metasenv subst metasenv,
+                    ugraph)
+    in
+    let u,typ,term, metasenv' =
+     let context_of_t, (t, metasenv, u) =
+      match terms_with_context, term with
+         [], None ->
+          raise
+           UnableToDetectTheTermThatMustBeGeneralizedYouMustGiveItExplicitly
+       | [], Some t -> context, t context metasenv u
+       | (context_of_t, _)::_, Some t -> 
+           context_of_t, t context_of_t metasenv u
+       | (context_of_t, t)::_, None -> context_of_t, (t, metasenv, u)
+     in
+      let t,subst,metasenv' =
+       try
+        CicMetaSubst.delift_rels [] metasenv
+         (List.length context_of_t - List.length context) t
+       with
+        CicMetaSubst.DeliftingARelWouldCaptureAFreeVariable ->
+         raise TheSelectedTermsMustLiveInTheGoalContext
+      in
+       (*CSC: I am not sure about the following two assertions;
+         maybe I need to propagate the new subst and metasenv *)
+       assert (subst = []);
+       assert (metasenv' = metasenv);
+       let typ,u = CicTypeChecker.type_of_aux' ~subst metasenv context t u in
+        u,typ,t,metasenv
+    in
+    (* We need to check:
+        1. whether they live in the context of the goal;
+           if they do they are also well-typed since they are closed subterms
+           of a well-typed term in the well-typed context of the well-typed
+           term
+        2. whether they are convertible
+    *)
+    ignore (
+     List.fold_left
+      (fun u (context_of_t,t) ->
+        (* 1 *)
+        let t,subst,metasenv'' =
+         try
+          CicMetaSubst.delift_rels [] metasenv'
+           (List.length context_of_t - List.length context) t
+         with
+          CicMetaSubst.DeliftingARelWouldCaptureAFreeVariable ->
+           raise TheSelectedTermsMustLiveInTheGoalContext in
+        (*CSC: I am not sure about the following two assertions;
+          maybe I need to propagate the new subst and metasenv *)
+        assert (subst = []);
+        assert (metasenv'' = metasenv');
+        (* 2 *)
+        let b,u1 = CicReduction.are_convertible ~subst context term t u in 
+         if not b then 
+          raise AllSelectedTermsMustBeConvertible
+         else
+          u1
+      ) u terms_with_context) ;
+    let status = (uri,metasenv',_subst,pbo,pty, attrs),goal in
+    let proof,goals =
+     PET.apply_tactic 
+      (T.thens 
+        ~start:
+          (cut_tac 
+           (C.Prod(
+             (mk_fresh_name_callback metasenv context C.Anonymous ~typ:typ), 
+             typ,
+             (ProofEngineReduction.replace_lifting_csc 1
+               ~equality:(==) 
+               ~what:(List.map snd terms_with_context)
+               ~with_what:(List.map (function _ -> C.Rel 1) terms_with_context)
+               ~where:ty)
+           )))
+        ~continuations:
+          [(apply_tac ~term:(C.Appl [C.Rel 1; CicSubstitution.lift 1 term])) ;
+            T.id_tac])
+        status
+    in
+     let _,metasenv'',_subst,_,_, _ = proof in
+      (* CSC: the following is just a bad approximation since a meta
+         can be closed and then re-opened! *)
+      (proof,
+        goals @
+         (List.filter
+           (fun j -> List.exists (fun (i,_,_) -> i = j) metasenv'')
+           (ProofEngineHelpers.compare_metasenvs ~oldmetasenv:metasenv
+             ~newmetasenv:metasenv')))
+ in
+  PET.mk_tactic (generalize_tac mk_fresh_name_callback ~pattern)
+;;
+
+let generalize_pattern_tac pattern =
+ let generalize_pattern_tac (proof,goal) =
+   let _,metasenv,_,_,_,_ = proof in
+   let conjecture = CicUtil.lookup_meta goal metasenv in
+   let _,context,_ = conjecture in 
+   let generalize_hyps =
+    let _,hpatterns,_ = ProofEngineHelpers.sort_pattern_hyps context pattern in
+     List.map fst hpatterns in
+   let ids_and_patterns =
+    List.map
+     (fun id ->
+       let rel,_ = ProofEngineHelpers.find_hyp id context in
+        id,(Some (fun ctx m u -> CicSubstitution.lift (List.length ctx - List.length context) rel,m,u), [], Some (ProofEngineTypes.hole))
+     ) generalize_hyps in
+   let tactics =
+    List.map
+     (function (id,pattern) ->
+       Tacticals.then_ ~start:(generalize_tac pattern)
+        ~continuation:(Tacticals.try_tactic
+          (ProofEngineStructuralRules.clear [id]))
+     ) ids_and_patterns
+   in
+    PET.apply_tactic (Tacticals.seq tactics) (proof,goal)
+ in
+  PET.mk_tactic (generalize_pattern_tac)
+;;
+
+let pattern_after_generalize_pattern_tac (tp, hpatterns, cpattern) =
+ let cpattern =
+  match cpattern with
+     None -> ProofEngineTypes.hole
+   | Some t -> t
+ in
+ let cpattern =
+  List.fold_left
+   (fun t (_,ty) -> Cic.Prod (Cic.Anonymous, ty, t)) cpattern hpatterns
+ in
+  tp, [], Some cpattern
+;;
+
 let elim_tac ?using ?(pattern = PET.conclusion_pattern None) term = 
- let elim_tac (proof, goal) =
-   let cpattern = match pattern with 
-      | None, [], Some cpattern -> cpattern
-      | _                       -> raise (PET.Fail (lazy "not implemented"))
-   in    
-   let ugraph = CicUniv.empty_ugraph in
+ let elim_tac pattern (proof, goal) =
+   let ugraph = CicUniv.oblivion_ugraph in
    let curi, metasenv, _subst, proofbo, proofty, attrs = proof in
    let conjecture = CicUtil.lookup_meta goal metasenv in
    let metano, context, ty = conjecture in 
+   let pattern = pattern_after_generalize_pattern_tac pattern in
+   let cpattern =
+    match pattern with 
+      | None, [], Some cpattern -> cpattern
+      | _ -> raise (PET.Fail (lazy "not implemented")) in    
     let termty,_ugraph = TC.type_of_aux' metasenv context term ugraph in
     let termty = CicReduction.whd context termty in
     let termty, metasenv', arguments, _fresh_meta =
@@ -655,7 +810,7 @@ let elim_tac ?using ?(pattern = PET.conclusion_pattern None) term =
        match ty_ty with
           C.Sort C.Prop -> "_ind"
         | C.Sort C.Set  -> "_rec"
-        | C.Sort C.CProp -> "_rec"
+        | C.Sort (C.CProp _) -> "_rect"
         | C.Sort (C.Type _)-> "_rect" 
         | C.Meta (_,_) -> raise TheTypeOfTheCurrentGoalIsAMetaICannotChooseTheRightElimiantionPrinciple
         | _ -> assert false
@@ -719,18 +874,40 @@ let elim_tac ?using ?(pattern = PET.conclusion_pattern None) term =
          (T.then_ ~start:(PET.mk_tactic (fun _ -> res)) ~continuation)
          dummy_status
    in
-   PET.mk_tactic elim_tac
+   let reorder_pattern ((proof, goal) as status) =
+     let _,metasenv,_,_,_,_ = proof in
+     let conjecture = CicUtil.lookup_meta goal metasenv in
+     let _,context,_ = conjecture in
+     let pattern = ProofEngineHelpers.sort_pattern_hyps context pattern in
+      PET.apply_tactic
+       (Tacticals.then_ ~start:(generalize_pattern_tac pattern)
+         ~continuation:(PET.mk_tactic (elim_tac pattern))) status
+   in
+    PET.mk_tactic reorder_pattern
 ;;
 
-let cases_intros_tac ?(howmany=(-1)) ?(mk_fresh_name_callback = FreshNamesGenerator.mk_fresh_name ~subst:[]) term =
- let cases_tac ~term (proof, goal) =
+let cases_intros_tac ?(howmany=(-1)) ?(mk_fresh_name_callback = FreshNamesGenerator.mk_fresh_name ~subst:[]) ?(pattern = PET.conclusion_pattern None) term =
+ let cases_tac pattern (proof, goal) =
   let module TC = CicTypeChecker in
   let module U = UriManager in
   let module R = CicReduction in
   let module C = Cic in
   let (curi,metasenv,_subst, proofbo,proofty, attrs) = proof in
   let metano,context,ty = CicUtil.lookup_meta goal metasenv in
-  let termty,_ = TC.type_of_aux' metasenv context term CicUniv.empty_ugraph in
+  let pattern = pattern_after_generalize_pattern_tac pattern in
+  let _cpattern =
+   match pattern with 
+     | None, [], Some cpattern ->
+        let rec is_hole =
+         function
+            Cic.Implicit (Some `Hole) -> true
+          | Cic.Prod (Cic.Anonymous,so,tgt) -> is_hole so && is_hole tgt
+          | _ -> false
+        in
+         if not (is_hole cpattern) then
+          raise (PET.Fail (lazy "not implemented"))
+     | _ -> raise (PET.Fail (lazy "not implemented")) in    
+  let termty,_ = TC.type_of_aux' metasenv context term CicUniv.oblivion_ugraph in
   let termty = CicReduction.whd context termty in
   let (termty,metasenv',arguments,fresh_meta) =
    TermUtil.saturate_term
@@ -744,7 +921,7 @@ let cases_intros_tac ?(howmany=(-1)) ?(mk_fresh_name_callback = FreshNamesGenera
     | _ -> raise NotAnInductiveTypeToEliminate
   in
   let paramsno,itty,patterns,right_args =
-    match CicEnvironment.get_obj CicUniv.empty_ugraph uri with
+    match CicEnvironment.get_obj CicUniv.oblivion_ugraph uri with
     | C.InductiveDefinition (tys,_,paramsno,_),_ ->
        let _,left_parameters,right_args = 
          List.fold_right 
@@ -833,7 +1010,7 @@ let cases_intros_tac ?(howmany=(-1)) ?(mk_fresh_name_callback = FreshNamesGenera
   let term_to_refine = C.MutCase (uri,typeno,outtype,term,patterns) in
   let refined_term,_,metasenv'',_ = 
     CicRefine.type_of_aux' metasenv' context term_to_refine
-      CicUniv.empty_ugraph
+      CicUniv.oblivion_ugraph
   in
   let new_goals =
     ProofEngineHelpers.compare_metasenvs
@@ -852,7 +1029,16 @@ let cases_intros_tac ?(howmany=(-1)) ?(mk_fresh_name_callback = FreshNamesGenera
     in
     proof'', patched_new_goals
   in
-  PET.mk_tactic (cases_tac ~term)
+   let reorder_pattern ((proof, goal) as status) =
+     let _,metasenv,_,_,_,_ = proof in
+     let conjecture = CicUtil.lookup_meta goal metasenv in
+     let _,context,_ = conjecture in
+     let pattern = ProofEngineHelpers.sort_pattern_hyps context pattern in
+      PET.apply_tactic
+       (Tacticals.then_ ~start:(generalize_pattern_tac pattern)
+         ~continuation:(PET.mk_tactic (cases_tac pattern))) status
+   in
+    PET.mk_tactic reorder_pattern
 ;;
 
 
@@ -873,22 +1059,3 @@ let elim_intros_simpl_tac ?(mk_fresh_name_callback = FreshNamesGenerator.mk_fres
        [ReductionTactics.simpl_tac
          ~pattern:(ProofEngineTypes.conclusion_pattern None)])
 ;;
-
-(* FG: insetrts a "hole" in the context (derived from letin_tac) *)
-
-let letout_tac =
-   let mk_fresh_name_callback = FreshNamesGenerator.mk_fresh_name ~subst:[] in
-   let term = C.Sort C.Set in
-   let letout_tac (proof, goal) =
-      let curi, metasenv, _subst, pbo, pty, attrs = proof in
-      let metano, context, ty = CicUtil.lookup_meta goal metasenv in
-      let newmeta = ProofEngineHelpers.new_meta_of_proof ~proof in
-      let fresh_name = mk_fresh_name_callback metasenv context (Cic.Name "hole") ~typ:term in
-      let context_for_newmeta = None :: context in
-      let irl = CicMkImplicit.identity_relocation_list_for_metavariable context_for_newmeta in
-      let newmetaty = CicSubstitution.lift 1 ty in
-      let bo' = C.LetIn (fresh_name, term, C.Meta (newmeta,irl)) in
-      let newproof, _ = ProofEngineHelpers.subst_meta_in_proof proof metano bo'[newmeta,context_for_newmeta,newmetaty] in
-      newproof, [newmeta]
-   in
-   PET.mk_tactic letout_tac
