@@ -23,5 +23,11 @@
  * http://helm.cs.unibo.it/
  *)
 
-val to_string: exn -> Stdpp.location option * string
+val compact_disambiguation_errors :
+  bool ->
+  (int * ((Stdpp.location list * string * string) list *
+   (DisambiguateTypes.domain_item * DisambiguateTypes.codomain_item) list *
+   Stdpp.location option * string Lazy.t * bool) list) list ->
+  (Stdpp.location option * (int list * (int list * (Stdpp.location list * string * string) list * (DisambiguateTypes.domain_item * DisambiguateTypes.codomain_item) list) list * string Lazy.t * bool) list) list
 
+val to_string: exn -> Stdpp.location option * string
